/*
 * bg_param.c
 * Copyright (C) 2014-2019 Peter Belkner <pbelkner@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <bg.h>

///////////////////////////////////////////////////////////////////////////////
static bg_param_unit_t bg_param_unit_ebu={
  .n.lu="LU",
  .n.lra="LRA",
  .n.sp="SP",
  .n.tp="TP",
#if defined (_WIN32) // [
  .w.lu=L"LU",
  .w.lra=L"LRA",
  .w.sp=L"SP",
  .w.tp=L"TP",
#endif // ]
};

static bg_param_unit_t bg_param_unit_db={
  .n.lu="dB",
  .n.lra="dB",
  .n.sp="dB",
  .n.tp="dB",
#if defined (_WIN32) // [
  .w.lu=L"dB",
  .w.lra=L"dB",
  .w.sp=L"dB",
  .w.tp=L"dB",
#endif // ]
};

///////////////////////////////////////////////////////////////////////////////
int bg_param_create(bg_param_t *param)
{
  param->print.vtbl=&bg_print_classic_vmt;
#if defined (FF_PRINTER) // [
  ff_printer_create(&param->printer,stdout);
#endif // ]
  param->loglevel=AV_LOG_QUIET;
  param->purged=0;
#if defined (_WIN32) // [
  param->codec.name[0]=L'\0';
#else // ] [
  param->codec.name=NULL;
#endif // ]
  param->temp_prefix=BG_TEMP_PREFIX;
  param->suppress.hierarchy=0;
  param->suppress.progress=0;
#if defined (_WIN32) // [
  // if LANG is set to e.g. "en_US.UTF-8" we assume we're run from
  // e.g. MSYS2 shell undestanding UTF-8 otherwise from MS console using
  // codepage OEM. In the latter case we need an OEM representation of
  // e.g. basename.
  param->oem=0;
#endif // ]
  param->decode.request.sample_fmt=-1;
  param->decode.request.channel_layout=-1ll;
  param->decode.drc.enabled=0;
  param->decode.drc.scale=0.0;
  param->result.f=stdout;
  param->output.dirname=NULL;
  param->ai=-1;
  param->vi=-1;
  param->flags.extension=0;
  param->flags.mode=0;
  param->flags.aggregate=BG_FLAGS_AGG_INTEGRATED;
  param->flags.norm=BG_FLAGS_NORM_EBU;
#if defined (BG_PARAM_DUMP) // [
  param->dump=0;
#endif // ]
#if defined (BG_PARAM_SEQUENTIAL) // [
  param->sequential=0;
#endif // ]
#if defined (BG_PARAM_STEREO) // [
  param->stereo=0;
#else // ] [
  param->decode.request.channel_layout=-1ll;
#endif // ]
  param->norm=-23.0;
  param->preamp=0.0;
  param->weight.enabled=0;
  param->weight.value=1.0;
  param->time=0;
#if defined (LIB1770_LFE) // [
  param->lfe=LIB1770_LFE;
#elif defined (BG_CHANNEL_LFE) // ] [
  param->lfe=BG_CHANNEL_LFE;
#else // ] [
  param->lfe=-1;
#endif // ]
#if defined (_WIN32) && defined (BG_WIN32_CREATE_LOCALE) // [
  param->locale=0;
#endif // ]
  param->overwrite=0;
  param->unit=&bg_param_unit_ebu;
  strncpy(param->tag.pfx,"REPLAYGAIN_",sizeof param->tag);
  param->out.sfx=NULL;
  param->ext.audio=FFL("flac");
  param->ext.video=FFL("mkv");
  param->interval.begin=-1ll;
  param->interval.duration=-1ll;
  ////////
  param->momentary.ms=400.0;
  param->momentary.partition=4;
  param->momentary.mean.gate=-10.0;
  param->momentary.range.gate=-20.0;
  param->momentary.range.lower_bound=0.1;
  param->momentary.range.upper_bound=0.95;
  ////////
  param->shortterm.ms=3000.0;
  param->shortterm.partition=3;
  param->shortterm.mean.gate=-10.0;
  param->shortterm.range.gate=-20.0;
  param->shortterm.range.lower_bound=0.1;
  param->shortterm.range.upper_bound=0.95;

  return 0;
}

void bg_param_destroy(bg_param_t *param FFUNUSED)
{
  ff_printer_destroy(&param->printer);
}

void bg_param_set_unit_ebu(bg_param_t *param)
{
  param->unit=&bg_param_unit_ebu;
}

void bg_param_set_unit_db(bg_param_t *param)
{
  param->unit=&bg_param_unit_db;
}
