/*
 * bg_root.h
 * Copyright (C) 2019 Peter Belkner <pbelkner@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <bg.h>

///////////////////////////////////////////////////////////////////////////////
static bg_tree_vtbl_t root_vtbl;

int bg_root_create(bg_tree_t *tree, bg_param_t *param)
{
  bg_album_t *album=&tree->album;

  /////////////////////////////////////////////////////////////////////////////
#if 0 // [
#if defined (_WIN32) // [
  // If writing to the console we need to write narrow characters otherwise
  // wide charcters.
  if (stdout==param->result.f||stderr==param->result.f)
    fputs("analyzing ...\n",param->result.f);
  else
    fputws(L"analyzing ...\n",param->result.f);
#else // ] [
  fputs("analyzing ...\n",param->result.f);
#endif // ]
#endif // ]

  /////////////////////////////////////////////////////////////////////////////
  if (bg_tree_common_create(tree,param,tree,NULL,NULL,&root_vtbl)<0) {
    DMESSAGE("creating tree");
    goto etree;
  }

  /////////////////////////////////////////////////////////////////////////////
  album->first=NULL;
  album->count=0ul;
#if defined (BG_PARAM_SEQUENTIAL) // [
  album->nleaves=0l;
#endif // ]

  /////////////////////////////////////////////////////////////////////////////
  return 0;
//cleanup:
  bg_tree_common_destroy(tree);
etree:
  return -1;
}

#if defined (_WIN32) // [
int bg_root_append(bg_tree_t *tree, ffchar_t *path)
#else // ] [
int bg_root_append(bg_tree_t *tree, const ffchar_t *path)
#endif // ]
{
  bg_tree_t **child=&tree->album.first;

  // iterate to the end of the list, i.e. append the path.
  while (*child)
    child=&(*child)->next;

#if defined (BG_TREE_CREATE_CHILD_WARNING) // [
  if (bg_child_create(child,tree->param,tree,tree,path)<0)
    BGVWARNING(BGL("creating child \"%s\""),path);
#else // ] [
  bg_child_create(child,tree->param,tree,tree,path);
#endif // ]

  return 0;
}

///////////////////////////////////////////////////////////////////////////////
static void bg_root_destroy(bg_tree_t *tree)
{
  bg_album_destroy(tree);
  bg_tree_common_destroy(tree);
}

static int bg_root_accept(bg_tree_t *tree, bg_visitor_t *vis)
{
  return vis->vtbl->dispatch_root(vis,tree);
}

static bg_tree_vtbl_t root_vtbl={
#if defined (PBU_DEBUG) // [
  .id="root",
#endif // ]
  .type=BG_TREE_TYPE_ROOT,
  .destroy=bg_root_destroy,
  .accept=bg_root_accept,
};
